//
//  MainPresenter.swift
//  SandboxApp
//
//  Created by Teko on 24/09/2021.
//

import Foundation
import JanusUI
import Janus
import Hestia

class MainPresenter: MainPresenterProtocol {
    
    weak var view: MainViewProtocol?
    var router: MainRouterProtocol
        
    init(view: MainViewProtocol, router: MainRouterProtocol) {
        self.view = view
        self.router = router
    }
    
    var scenarios: [Scenario] = [
        .openAppWithoutLogin,
        .openAppWithLogin,
        .payment
    ]
    
    var appTypes: [MiniAppType] = [
        .native,
        .reactNative,
        .web
    ]
    
    var appCode: String = AppPreferences.shared.lastOpenedAppCode ?? MiniApp.tc1.appCode
    var selectedScenario: Scenario = .openAppWithoutLogin
    var selectedAppType: MiniAppType = .native
//    var requiredAppCode: Bool { selectedScenario != .openAppWithoutLogin }
    
    var isLoggedIn: Bool { terraAuth?.isAuthorized ?? false }
    
    func selectScenario(at index: Int) {
        selectedScenario = scenarios[index]
        
        if AppPreferences.shared.lastOpenedAppCode == nil {
            resetDefaultAppCode()
        }
        
        view?.updateUI()
    }
    
    func selectAppType(at index: Int) {
        selectedAppType = appTypes[index]
    }
    
    func appCodeChanged(appCode: String) {
        AppPreferences.shared.lastOpenedAppCode = appCode
        self.appCode = appCode
    }
    
    func didTapOpenApp() {
        openSelectedMiniApp()
    }
    
    func didTapLogin() {
        if isLoggedIn {
            terraAuth?.logoutGoogle()
            terraAuth?.logout()
            view?.reloadAuthState()
        } else {
            router.openLoginUI(delegate: self)
        }
    }
    
    func didTapDefault() {
        resetDefaultAppCode()
    }
    
    func resetDefaultAppCode() {
        AppPreferences.shared.lastOpenedAppCode = nil
        let defaultAppCode = getDefaultAppCode(forScenario: selectedScenario)
        self.appCode = defaultAppCode
        view?.fillAppCode(appCode: defaultAppCode)
    }
    
    private func openSelectedMiniApp() {
        router.openMiniApp(
            appType: selectedAppType,
            scenario: selectedScenario,
            appCode: appCode,
            delegate: self
        )
    }
    
    private func getDefaultAppCode(forScenario scenario: Scenario) -> String {
        switch scenario {
        case .openAppWithoutLogin:
            return MiniApp.tc1.appCode
        case .openAppWithLogin:
            return MiniApp.tc2.appCode
        case .payment:
            return MiniApp.tc3.appCode
        }
    }
    
}

extension MainPresenter: LoginUIDelegate {
    
    func onLoginSuccess(authInfo: IAuthInfo) {
        view?.reloadAuthState()
    }
    
    func onLoginCancelled() {
        
    }
    
}

extension MainPresenter: HestiaCallback {
    
    func onSuccess() {
        
    }
    
    func onError(_ error: HestiaError) {
        switch error {
        case .exchangeTokenFailed:
            if !isLoggedIn {
                router.openLoginUI(delegate: LoginDelegate(loginSuccess: { [weak self] in
                    self?.openSelectedMiniApp()
                }, loginCancelled: nil))
            } else {
                view?.showAlert(message: error.localizedDescription)
            }
        case .appNotFound:
            view?.showAlert(message: Strings.appNotFoundMessage)
        case .connectorNotIntegrated:
            view?.showAlert(message: Strings.connectorNotIntegratedMessage)
        default:
            view?.showAlert(message: error.rawValue)
        }
    }
    
}

class LoginDelegate: LoginUIDelegate {
    
    var loginSuccess: (() -> ())?
    var loginCancelled: (() -> ())?
    
    init(loginSuccess: (() -> ())?, loginCancelled: (() -> ())?) {
        self.loginSuccess = loginSuccess
        self.loginCancelled = loginCancelled
    }
    
    func onLoginSuccess(authInfo: IAuthInfo) {
        loginSuccess?()
    }
    
    func onLoginCancelled() {
        loginCancelled?()
    }
    
}
