//
//  MainViewController.swift
//  SanboxApp
//
//  Created by Teko on 23/09/2021.
//

import UIKit
import Apollo

class MainViewController: UIViewController, MainViewProtocol {

    @IBOutlet weak var versionLbl: ApolloLabel!
    @IBOutlet weak var greetingLbl: ApolloLabel!
    @IBOutlet weak var scenarioLbl: ApolloLabel!
    @IBOutlet weak var appTypeLbl: ApolloLabel!
    @IBOutlet weak var scenarioDropdown: ApolloDropdown!
    @IBOutlet weak var appTypeDropdown: ApolloDropdown!
    @IBOutlet weak var appCodeLbl: ApolloLabel!
    @IBOutlet weak var appCodeField: ApolloFormField!
    @IBOutlet weak var openBtn: ApolloButton!
    @IBOutlet weak var loginBtn: ApolloButton!
    
    @IBOutlet weak var appCodeFieldBottom: NSLayoutConstraint!
    @IBOutlet weak var appTypeDropdownBottom: NSLayoutConstraint!
    
    var presenter: MainPresenterProtocol?
    
    var isAppCodeRequired: Bool { true }
    
    override func viewDidLoad() {
        super.viewDidLoad()

        setupLabels()
        setupDropdowns()
        setupButtons()
        setupTextFields()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        reloadAuthState()
        setupNavigationBar()
    }
    
    func setupNavigationBar() {
        navigationController?.setNavigationBarHidden(true, animated: true)
    }
    
    func setupLabels() {
        greetingLbl.style = .subtitle2
        
        scenarioLbl.style = .subtitle4
        appTypeLbl.style = .subtitle4
        
        versionLbl.style = .body1
        versionLbl.text = String(format: Strings.appVersion, Bundle.main.version)
    }
    
    func setupDropdowns() {
        scenarioDropdown.dropdownDataSource = ScenarioDropdownDataSource(scenarios: presenter?.scenarios ?? [])
        scenarioDropdown.dropdownDelegate = self
        scenarioDropdown.text = presenter?.selectedScenario.title
        
        appTypeDropdown.dropdownDataSource = AppTypesDatasource(appTypes: presenter?.appTypes ?? [])
        appTypeDropdown.dropdownDelegate = self
        appTypeDropdown.text = presenter?.selectedAppType.title
    }
    
    func setupButtons() {
        openBtn.size = .large
        openBtn.addTarget(self, action: #selector(handleOpenTapped), for: .touchUpInside)
        
        loginBtn.size = .large
        loginBtn.addTarget(self, action: #selector(handleLoginTapped), for: .touchUpInside)
    }
    
    func setupTextFields() {
        appCodeField.delegate = self
        appCodeField.text = presenter?.appCode
    }
    
    func updateUI() {
        UIView.animate(withDuration: 0.2) {
            self.appCodeLbl.isHidden = !self.isAppCodeRequired
            self.appCodeField.isHidden = !self.isAppCodeRequired
            self.appCodeFieldBottom.isActive = self.isAppCodeRequired
            self.appTypeDropdownBottom.isActive = !self.isAppCodeRequired
            self.view.layoutIfNeeded()
        }
    }
    
    func reloadAuthState() {
        if presenter?.isLoggedIn ?? false {
            loginBtn.title = Strings.logout
        } else {
            loginBtn.title = Strings.login
        }
    }
    
    @objc func handleOpenTapped() {
        presenter?.didTapOpenApp()
    }
    
    @objc func handleLoginTapped() {
        presenter?.didTapLogin()
    }
    
    @IBAction func defaultButtonTapped(_ sender: Any) {
        presenter?.didTapDefault()
    }
    
    func fillAppCode(appCode: String) {
        appCodeField.text = appCode
    }
    
    
}

extension MainViewController: FormFieldDelegate {
    
    func formField(_ formField: ApolloFormField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        if let text = formField.text as NSString? {
            let newString = text.replacingCharacters(in: range, with: string)
            presenter?.appCodeChanged(appCode: newString)
        }
        return true
    }
    
}

extension MainViewController: DropdownDelegate {
    
    func dropdown(_ dropdown: ApolloDropdown, didSelectRowAt index: Int) {
        switch dropdown {
        case scenarioDropdown:
            presenter?.selectScenario(at: index)
        case appTypeDropdown:
            presenter?.selectAppType(at: index)
        default: ()
        }
    }
    
}
